#!/usr/bin/env python

'''
    a DMX512 controlled image display
    receive a control byte via UART
    uses external hardware for channel filtering!

'''

# python standard packages
import io
import os
import platform
import time

# add on packages
import PySimpleGUI as sg
from PIL import Image, ImageTk, ImageOps
import serial

# ------------------------------------------------------------------------------
# use PIL to read data of one image
# ------------------------------------------------------------------------------

def get_img_data(f_name, maxsize=(1920, 1080), first=False):
    """Generate image data using PIL
    """
    img = Image.open(f_name)
    #img.thumbnail(maxsize, resample = Image.BILINEAR)
    img = ImageOps.contain(img, maxsize, method = Image.BILINEAR)
    if first:                     # tkinter is inactive the first time
        bio = io.BytesIO()
        img.save(bio, format="PNG")
        del img
        return bio.getvalue()
    return ImageTk.PhotoImage(img)

# ------------------------------------------------------------------------------
# Main start
# ------------------------------------------------------------------------------

if platform.system() == 'Windows':
    folder = 'C:/tmp/DMX512_PICS'
    filename = 'C:/tmp/DMX512_PICS/bereit.png'
    com_name = 'COM10'
elif platform.system() == 'Linux':
    folder = '/media/usb/DMX512_PICS'
    #folder = '/home/pi/Desktop/DMX512_PICS'    #TEST
    filename = '/home/pi/Documents/bereit.png'
    com_name = '/dev/serial0'
else:
    sg.popup('Unknown operation system!')
    raise SystemExit()

# list ports
# python -m serial.tools.list_ports
my_serial = serial.Serial(com_name, baudrate=9600, timeout=0, write_timeout=0.1)  # open serial port
print("Serial port " + my_serial.name)
#my_serial.write(b'Hello World\r\n');

# supported image types
img_types = (".png", ".jpg", "jpeg", ".bmp")

fnames = []
num_files = 0
for i in range(0, 256):
    for j in img_types:
        name = "{:03d}".format(i) + j
        path = os.path.join(folder, name)
        if os.path.exists(path) and os.path.isfile(path):
            break;
    #print(path)
    if os.path.exists(path) and os.path.isfile(path):
        fnames.append(name)
        num_files +=1
        print(name)
    else:
        fnames.append('')

if num_files == 0:
    sg.popup('Keine Bilder im Ordner ' + folder + ' !')
    raise SystemExit()

#print(fnames)
sg.theme('Black')

image_element1 = sg.Image(data=get_img_data(filename, first=True), background_color = 'Black')
layout1 = [[image_element1]]

window1 = sg.Window('DMX512', layout1, size=(1920, 1080), no_titlebar = True,
                    return_keyboard_events=True, finalize=True, element_justification = "center")
window1.set_cursor("none")
window1.bring_to_front()
window1.force_focus()

old_pic, new_pic = 0, 0

# main loop

while True:

    event, values = window1.read(timeout=20)
                            
    if event == sg.WIN_CLOSED or event[0:6] == 'Escape':
        break
    elif event is sg.TIMEOUT_KEY:
        #print("check UART")
        serialRX = my_serial.read(size=1);
        if len(serialRX) == 0: continue
        new_pic = int.from_bytes(serialRX, byteorder = 'big')
        #TEST
            #new_pic += 1
            #if new_pic > 15: new_pic = 0
            #print(new_pic)
        if old_pic != new_pic:
            old_pic = new_pic
            filename = os.path.join(folder, fnames[new_pic])
            # update window with new image
            if len(fnames[new_pic])==0:
                image_element1.update('')
            else:
                image_element1.update(data=get_img_data(filename))
            window1.Refresh()
            
window1.close()
my_serial.close()
